"""
Filter that adds a Location header to a response if the status is 201 Created.
"""

__all__ = ['locationfilter']

from twisted.web2 import responsecode
import urlparse

def addlocation(request, location):
    """
    Ensure that the supplied location URI is added to any response to this request.

    @param request:  L{IRequest} the request whose response is to be modified.
    @param location: the C{str} containing the path relative URI for the location header value.
    """

    def locationfilter(request, response): #@UnusedVariable
        """
        Add a Location header to the response if the status is 201. Note that the location
        value must be a valid URI (already % encoded etc).
    
        @param request:  L{IRequest} for the current request.
        @param response: L{IResponse} for the response to add the header to.
        @return: the original L{IResponse} with (or without) the added header.
        """
        
        if (response.code == responsecode.CREATED):
            # Check to see whether we have an absolute URI or not. If not, have the request
            # turn it into an absolute URI.
            (scheme, host, path, params, querystring, fragment) = urlparse.urlparse(location)
            if scheme == '':
                abslocation = request.unparseURL(path=location)
            else:
                abslocation = location
        
            response.headers.setHeader('location', abslocation)
        return response

    request.addResponseFilter(locationfilter)
    return
